@@ -1,5 +1,4 @@
-//go:build linux || freebsd
-// +build linux freebsd
+//go:build !remote && (linux || freebsd)
 
 package libpod
 
@@ -12,11 +11,12 @@ import (
 	"strings"
 	"time"
 
-	"github.com/containers/podman/v4/libpod/define"
-	"github.com/containers/podman/v4/libpod/events"
-	volplugin "github.com/containers/podman/v4/libpod/plugin"
+	"github.com/containers/podman/v5/libpod/define"
+	"github.com/containers/podman/v5/libpod/events"
+	volplugin "github.com/containers/podman/v5/libpod/plugin"
 	"github.com/containers/storage"
 	"github.com/containers/storage/drivers/quota"
+	"github.com/containers/storage/pkg/fileutils"
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/containers/storage/pkg/stringid"
 	pluginapi "github.com/docker/go-plugins-helpers/volume"
@@ -54,12 +54,18 @@ func (r *Runtime) newVolume(ctx context.Context, noCreatePluginVolume bool, opti
 	volume.config.CreatedTime = time.Now()
 
 	// Check if volume with given name exists.
-	if !volume.ignoreIfExists {
-		exists, err := r.state.HasVolume(volume.config.Name)
-		if err != nil {
-			return nil, fmt.Errorf("checking if volume with name %s exists: %w", volume.config.Name, err)
-		}
-		if exists {
+	exists, err := r.state.HasVolume(volume.config.Name)
+	if err != nil {
+		return nil, fmt.Errorf("checking if volume with name %s exists: %w", volume.config.Name, err)
+	}
+	if exists {
+		if volume.ignoreIfExists {
+			existingVolume, err := r.state.Volume(volume.config.Name)
+			if err != nil {
+				return nil, fmt.Errorf("reading volume from state: %w", err)
+			}
+			return existingVolume, nil
+		} else {
 			return nil, fmt.Errorf("volume with name %s already exists: %w", volume.config.Name, define.ErrVolumeExists)
 		}
 	}
@@ -79,7 +85,7 @@ func (r *Runtime) newVolume(ctx context.Context, noCreatePluginVolume bool, opti
 			switch strings.ToLower(key) {
 			case "device":
 				if strings.ToLower(volume.config.Options["type"]) == define.TypeBind {
-					if _, err := os.Stat(val); err != nil {
+					if err := fileutils.Exists(val); err != nil {
 						return nil, fmt.Errorf("invalid volume option %s for driver 'local': %w", key, err)
 					}
 				}
@@ -172,28 +178,35 @@ func (r *Runtime) newVolume(ctx context.Context, noCreatePluginVolume bool, opti
 		if err := LabelVolumePath(fullVolPath, volume.config.MountLabel); err != nil {
 			return nil, err
 		}
-		if volume.config.DisableQuota {
+		switch {
+		case volume.config.DisableQuota:
 			if volume.config.Size > 0 || volume.config.Inodes > 0 {
 				return nil, errors.New("volume options size and inodes cannot be used without quota")
 			}
-		} else {
+		case volume.config.Options["type"] == define.TypeTmpfs:
+			// tmpfs only supports Size
+			if volume.config.Inodes > 0 {
+				return nil, errors.New("volume option inodes not supported on tmpfs filesystem")
+			}
+		case volume.config.Inodes > 0 || volume.config.Size > 0:
 			projectQuotaSupported := false
 			q, err := quota.NewControl(r.config.Engine.VolumePath)
 			if err == nil {
 				projectQuotaSupported = true
 			}
-			quota := quota.Quota{}
-			if volume.config.Size > 0 || volume.config.Inodes > 0 {
-				if !projectQuotaSupported {
-					return nil, errors.New("volume options size and inodes not supported. Filesystem does not support Project Quota")
-				}
-				quota.Size = volume.config.Size
-				quota.Inodes = volume.config.Inodes
+			if !projectQuotaSupported {
+				return nil, errors.New("volume options size and inodes not supported. Filesystem does not support Project Quota")
 			}
-			if projectQuotaSupported {
-				if err := q.SetQuota(fullVolPath, quota); err != nil {
-					return nil, fmt.Errorf("failed to set size quota size=%d inodes=%d for volume directory %q: %w", volume.config.Size, volume.config.Inodes, fullVolPath, err)
-				}
+			quota := quota.Quota{
+				Inodes: volume.config.Inodes,
+				Size:   volume.config.Size,
+			}
+			// Must use volPathRoot not fullVolPath, as we need the
+			// base path for the volume - without the `_data`
+			// subdirectory - so the quota ID assignment logic works
+			// properly.
+			if err := q.SetQuota(volPathRoot, quota); err != nil {
+				return nil, fmt.Errorf("failed to set size quota size=%d inodes=%d for volume directory %q: %w", volume.config.Size, volume.config.Inodes, fullVolPath, err)
 			}
 		}
 
@@ -219,13 +232,6 @@ func (r *Runtime) newVolume(ctx context.Context, noCreatePluginVolume bool, opti
 
 	// Add the volume to state
 	if err := r.state.AddVolume(volume); err != nil {
-		if volume.ignoreIfExists && errors.Is(err, define.ErrVolumeExists) {
-			existingVolume, err := r.state.Volume(volume.config.Name)
-			if err != nil {
-				return nil, fmt.Errorf("reading volume from state: %w", err)
-			}
-			return existingVolume, nil
-		}
 		return nil, fmt.Errorf("adding volume to state: %w", err)
 	}
 	defer volume.newVolumeEvent(events.Create)
@@ -355,13 +361,11 @@ func (r *Runtime) removeVolume(ctx context.Context, v *Volume, force bool, timeo
 		return define.ErrVolumeRemoved
 	}
 
-	v.lock.Lock()
-	defer v.lock.Unlock()
-
-	// Update volume status to pick up a potential removal from state
-	if err := v.update(); err != nil {
-		return err
-	}
+	// DANGEROUS: Do not lock here yet because we might needed to remove containers first.
+	// In general we must always acquire the ctr lock before a volume lock so we cannot lock.
+	// THIS MUST BE DONE to prevent ABBA deadlocks.
+	// It also means the are several races around creating containers with volumes and removing
+	// them in parallel. However that problem exists regadless of taking the lock here or not.
 
 	deps, err := r.state.VolumeInUse(v)
 	if err != nil {
@@ -398,6 +402,15 @@ func (r *Runtime) removeVolume(ctx context.Context, v *Volume, force bool, timeo
 		}
 	}
 
+	// Ok now we are good to lock.
+	v.lock.Lock()
+	defer v.lock.Unlock()
+
+	// Update volume status to pick up a potential removal from state
+	if err := v.update(); err != nil {
+		return err
+	}
+
 	// If the volume is still mounted - force unmount it
 	if err := v.unmount(true); err != nil {
 		if force {
